module ConfigCore
  class Scope
    attr_reader :name

    def initialize(name)
      @name = name
      @parameters = {}
    end

    def empty?
      @parameters.empty?
    end

    def has_parameter?(name)
      @parameters.has_key?(name)
    end

    def define(name, value, editable: true)
      raise ParameterAlreadyExistsError if has_parameter?(name)

      parameter = Parameter.new(name, value)
      parameter.disable_editing! unless editable

      @parameters[name] = parameter

      define_parameter_reader(name, parameter)
      define_parameter_value_reader(name, parameter)
      define_parameter_value_writer(name, parameter)

      return parameter
    end

    private
    def define_parameter_reader(name, parameter)
      define_singleton_method(:"#{name}_parameter") { parameter }
    end

    def define_parameter_value_reader(name, parameter)
      define_singleton_method(name) { parameter.value }
    end

    def define_parameter_value_writer(name, parameter)
      define_singleton_method(:"#{name}=") { |value| parameter.value = value }
    end
  end
end
